/* **********************************************************************************
*
* Copyright (c) Microsoft Corporation. All rights reserved.
*
* This source code is subject to terms and conditions of the Shared Source License
* for DSL Editor PowerToy. A copy of the license can be found in the License.htm file
* at the root of this distribution. If you can not locate the Shared Source License
* for DSL Editor PowerToy, please obtain a copy from: http://www.codeplex.com/dsltreegrideditor/Project/License.aspx.
* By using this source code in any fashion, you are agreeing to be bound by
* the terms of the Shared Source License for DSL Editor PowerToy.
*
* You must not remove this notice, or any other, from this software.
*
* **********************************************************************************/
using System;
using System.ComponentModel;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Text;
using System.Windows.Forms;

using VSTrid = Microsoft.VisualStudio.VirtualTreeGrid;

using DslEditorPowerToy.Controls.VirtualTreeGrid;
using DslEditorPowerToy.Controls.VirtualTreeGrid.Branches;
using DslEditorPowerToy.Controls.VirtualTreeGrid.Validation;

using DslEditorPowerToy.Controls.Editors.Special.DomainModelTreeGrid;
using DslEditorPowerToy.Controls.Editors.Special.DomainModelTreeGrid.Data;

namespace DslEditorPowerToy.Controls.Editors.Special.DomainModelTreeGrid.Branches
{
    /// <summary>
    /// Provides a branch representing the contents of a domain class
    /// This branch adds and manages rows for domain relationships
    /// </summary>
    internal class DomainClassBranch : DomainElementContainerBranch
    {
        #region Constructors
        public DomainClassBranch(VirtualTreeGridControl treeControl,
            IDomainClass branchData, Type branchDataItemType)
            : base(treeControl, branchData, branchDataItemType)
        {
        }
        #endregion

        #region IBranch Members
        /// <summary>
        /// Returns whether to allow expansion of row
        /// </summary>
        /// <param name="row"></param>
        /// <param name="column"></param>
        /// <returns></returns>
        public override bool IsExpandable(int row, int column)
        {
            DomainElementCollection<IDomainElement> elements = this.GetElementRows();
            if (row >= elements.Count)
                return false;

            IDomainRelationship relationship = elements[row] as IDomainRelationship;
            if ((null != relationship.ChildClasses)
                && (relationship.ChildClasses.Count > 0))
                return true;
            else
                return false;
        }

        /// <summary>
        /// Locates specified object in branch
        /// </summary>
        /// <param name="obj"></param>
        /// <param name="style"></param>
        /// <param name="locateOptions"></param>
        /// <returns></returns>
        public override VSTrid.LocateObjectData LocateObject(object obj, VSTrid.ObjectStyle style, int locateOptions)
        {
            switch (style)
            {
                case VSTrid.ObjectStyle.TrackingObject:
                    //Since, the items are only accessible in IList by index, we can return by index
                    if (obj is int)
                        return new VSTrid.LocateObjectData((int)obj, 0, (int)VSTrid.TrackingObjectAction.ThisLevel);
                    
                    //Check if of type somewhere under this branch
                    IDomainRelationship relationship = null;
                    if (obj is IDomainClass)
                        relationship = ((IDomainClass)obj).ParentRelationship;
                    if (obj is IDomainRelationship)
                        relationship = obj as IDomainRelationship;

                    //Find ancestor relation at this level
                    while (null != relationship)
                    {
                        //Check if ancestor at this level
                        int index = this.BranchData.ChildRelationships.IndexOf(relationship);
                        if (-1 != index)
                        {
                            //Determine if at this level or a sub level from this level
                            if (obj == relationship)
                                return new VSTrid.LocateObjectData(index, 0, (int)VSTrid.TrackingObjectAction.ThisLevel);
                            else
                                return new VSTrid.LocateObjectData(index, 0, (int)VSTrid.TrackingObjectAction.NextLevel);
                        }

                        //Move up to grand parent relationship
                        if (null == relationship.ParentClass)
                            relationship = null;
                        else
                            relationship = relationship.ParentClass.ParentRelationship;
                    }
                    return new VSTrid.LocateObjectData(0, 0, (int)VSTrid.TrackingObjectAction.NotTracked);

                default:
                    break;
            }

            return base.LocateObject(obj, style, locateOptions);
        }
        /// <summary>
        /// Returns the object at specified position and type
        /// </summary>
        /// <param name="row"></param>
        /// <param name="cellColumnIndex"></param>
        /// <param name="style"></param>
        /// <param name="options"></param>
        /// <returns></returns>
        public override object GetObject(int row, int column, VSTrid.ObjectStyle style, ref int options)
        {
            switch (style)
            {
                case VSTrid.ObjectStyle.ExpandedBranch:
                    IDomainRelationship relationship = GetElementAtRow(row) as IDomainRelationship;
                    if ((null != relationship.ClassType)
                        && (null != relationship.ChildClasses))
                        return new DomainRelationshipBranch(base.TreeControl, relationship, relationship.ClassType);
                    else
                        return null;

                case VSTrid.ObjectStyle.TrackingObject:
                    //Only want to track the row (and ignore the column)
                    //Since IDesignerElement only supports IList of items, we can only reference by index
                    return GetElementAtRow(row);

                default:
                    return null;
            }
        }

        #endregion

        #region Protected Properties
        protected new IDomainClass BranchData
        {
            get
            {
                return base.BranchData as IDomainClass;
            }
        }
        #endregion

        #region BranchBase Members
        protected override short GetDataItemImageIndex(int row, int column)
        {
            if (0 == column)
                return BranchItemGlyphs.DomainRelationship;
            else
                return -1;
        }
        #endregion

        #region DomainElementBranch Methods
        /// <summary>
        /// Returns the row dataItem for this branch
        /// </summary>
        /// <returns></returns>
        protected override DomainElementCollection<IDomainElement> GetElementRows()
        {
            if (null != this.BranchData.ChildRelationships)
                return this.BranchData.ChildRelationships.ConvertAll();
            else
                return null;
        }
        #endregion
    }
}
