/* **********************************************************************************
*
* Copyright (c) Microsoft Corporation. All rights reserved.
*
* This source code is subject to terms and conditions of the Shared Source License
* for DSL Editor PowerToy. A copy of the license can be found in the License.htm file
* at the root of this distribution. If you can not locate the Shared Source License
* for DSL Editor PowerToy, please obtain a copy from: http://www.codeplex.com/dsltreegrideditor/Project/License.aspx.
* By using this source code in any fashion, you are agreeing to be bound by
* the terms of the Shared Source License for DSL Editor PowerToy.
*
* You must not remove this notice, or any other, from this software.
*
* **********************************************************************************/
using System;
using System.Collections.Specialized;
using System.IO;
using System.ComponentModel.Design;

using Microsoft.Practices.Common.Services;
using Microsoft.Practices.ComponentModel;
using Microsoft.Practices.RecipeFramework;
using Microsoft.Practices.RecipeFramework.Library;
using Microsoft.Practices.RecipeFramework.Services;

using EnvDTE;

namespace DslEditorPowerToy.Recipes.Actions
{
    public abstract class ActionBase : ConfigurableAction
    {
        private EnvDTE.DTE _dte = null;
        private ExpressionEvaluationService _evaluator = null;
        private StringDictionary _attributes = null;

        #region Constructors
        protected ActionBase()
            : base()
        {
            this._evaluator = new ExpressionEvaluationService();
        }
        #endregion

        #region Protected Properties
        /// <summary>
        /// Gets the visual studio automation object
        /// </summary>
        [CLSCompliant(false)]
        protected EnvDTE.DTE DTE
        {
            get
            {
                //Test if already accessed
                if (null == this._dte)
                {
                    //Get visual studio automation object (we cant do this in the ctor)
                    this._dte = (EnvDTE.DTE)GetService(typeof(EnvDTE.DTE));
                }
                return this._dte;
            }
        }
        /// <summary>
        /// Gets this guidance package
        /// </summary>
        protected Microsoft.Practices.RecipeFramework.Configuration.GuidancePackage Package
        {
            get
            {
                IConfigurationService configurationService = (IConfigurationService)this.GetService<IConfigurationService>();
                return configurationService.CurrentPackage;
            }
        }
        /// <summary>
        /// Gets path to this guidance package
        /// </summary>
        protected string PackagePath
        {
            get
            {
                TypeResolutionService resolution = (TypeResolutionService)this.GetService<ITypeResolutionService>();
                return resolution.BasePath;
            }
        }        /// <summary>
        /// Returns the path to this guidance package 'templates' directory
        /// </summary>
        protected string PackageTemplatePath
        {
            get
            {
                return Path.Combine(this.PackagePath, "Templates");
            }
        }
        protected StringDictionary ConfiguredAttributes
        {
            get
            {
                return this._attributes;
            }
        }
        #endregion

        #region Protected Methods
        /// <summary>
        /// Validates the parameters of the action before execution
        /// </summary>
        protected abstract void ValidateParameters();
        /// <summary>
        /// Returns the full path of a the template in the package
        /// </summary>
        /// <returns></returns>
        protected string BuildPackageTemplatePath(string relTemplatePath)
        {
            return Path.Combine(this.PackageTemplatePath, relTemplatePath);
        }
        /// <summary>
        /// Evaluates the specified parameter using dictionary of all arguments of the template
        /// </summary>
        /// <param name="argument"></param>
        /// <returns></returns>
        protected object Evaluate(string parameter)
        {
            //Get all arguments
            IDictionaryService dictionaryService = (IDictionaryService)this.GetService(typeof(IDictionaryService));

            //Evaluate
            return this._evaluator.Evaluate(parameter, new ServiceAdapterDictionary(dictionaryService));
        }
        #endregion

        #region Public Methods
        /// <summary>
        /// Called to execture recipe
        /// </summary>
        public override void Execute()
        {
            //Fixup for a GAX bug to allow recursive calls of same recipe
            IConfigurationService configService = (IConfigurationService)this.GetService(typeof(IConfigurationService));
            if (null != configService)
            {
                if (null != configService.CurrentRecipe)
                {
                    if (null != configService.CurrentRecipe.GatheringServiceData)
                    {
                        if (null == configService.CurrentRecipe.GatheringServiceData.Any)
                            configService.CurrentRecipe.GatheringServiceData = null;
                    }
                }
            }
        }
        public override void Configure(System.Collections.Specialized.StringDictionary attributes)
        {
            base.Configure(attributes);
            this._attributes = attributes;
        }
        #endregion
    }
}
